@@ -4,8 +4,8 @@
 #' @description Calculate p-values for single coefficient and multiple contrast
 #'   hypothesis tests using cluster wild bootstrapping.
 #'
-#' @param full_model Model fit using \code{robumeta::robu()} and
-#'   \code{metafor::rma.mv()} that includes the full set of moderators in the
+#' @param full_model Model fit using \code{robumeta::robu()},
+#'   \code{metafor::rma.mv()}, or \code{metafor::rma.uni()} that includes the full set of moderators in the
 #'   meta-regression model.
 #' @param constraints A q X p constraint matrix be tested. Alternately, a
 #'   function to create such a matrix, specified using
@@ -81,7 +81,7 @@ Wald_test_cwb <- function(full_model,
   future_f_args <- if (inherits(full_model,"rma")) {
     list(
       future.packages = c("clubSandwich","metafor"),
-      future.envir = attr(full_model$random[[1]], ".Environment")
+      future.envir = find_env(full_model)
     )
   } else {
     NULL
