@@ -56,53 +56,67 @@ mod_F <- rma.mv(yi ~ Crit.Cat + Crit.Domain + IAT.Focus, V = V,
                 data = oswald2013,
                 sparse = TRUE)
 
+mod_Y <- rma.mv(yi ~ Crit.Cat + Crit.Domain + IAT.Focus + Scoring, V = V,
+                 data = oswald2013,
+                 sparse = TRUE)
+
+mod_Z <- rma.mv(yi ~ Crit.Cat + Crit.Domain + IAT.Focus, V = V,
+                data = oswald2013,
+                sparse = TRUE)
+
 Cmat_A <- constrain_equal("Crit.Cat", reg_ex = TRUE, coef(mod_B))
 Cmat_B <- constrain_zero(7:10, coef(mod_C1))
 Cmat_D <- constrain_zero("Crit.Cat", reg_ex = TRUE, coef(mod_C1))
 Cmat_E <- constrain_zero("Crit.Domain", reg_ex = TRUE, coef(mod_C2))
 Cmat_F <- constrain_zero("Scoring", reg_ex = TRUE, coef(mod_C2))
 
+tol <- 1e-6
+
 test_that("estimate_null() works for rma.mv objects.", {
 
   mod_A_con <- estimate_null(mod_B, C_mat = Cmat_A)
-  expect_equal(get_fitted(mod_A), get_fitted(mod_A_con))
-  expect_equal(get_res(mod_A), get_res(mod_A_con))
+  expect_equal(get_fitted(mod_A), get_fitted(mod_A_con), tolerance = tol)
+  expect_equal(get_res(mod_A), get_res(mod_A_con), tolerance = tol)
 
   mod_D_con <- estimate_null(mod_C1, Cmat_D)
-  expect_equal(get_fitted(mod_D), get_fitted(mod_D_con))
-  expect_equal(get_res(mod_D), get_res(mod_D_con))
+  expect_equal(get_fitted(mod_D), get_fitted(mod_D_con), tolerance = tol)
+  expect_equal(get_res(mod_D), get_res(mod_D_con), tolerance = tol)
 
   mod_E_con <- estimate_null(mod_C2, Cmat_E)
-  expect_equal(get_fitted(mod_E), get_fitted(mod_E_con))
-  expect_equal(get_res(mod_E), get_res(mod_E_con))
+  expect_equal(get_fitted(mod_E), get_fitted(mod_E_con), tolerance = tol)
+  expect_equal(get_res(mod_E), get_res(mod_E_con), tolerance = tol)
 
   mod_F_con <- estimate_null(mod_C2, Cmat_F)
-  expect_equal(get_fitted(mod_F), get_fitted(mod_F_con))
-  expect_equal(get_res(mod_F), get_res(mod_F_con))
+  expect_equal(get_fitted(mod_F), get_fitted(mod_F_con), tolerance = tol)
+  expect_equal(get_res(mod_F), get_res(mod_F_con), tolerance = tol)
 
   mod_B_con <- estimate_null(mod_C1, Cmat_B)
-  expect_equal(get_fitted(mod_B), get_fitted(mod_B_con))
-  expect_equal(get_res(mod_B), get_res(mod_B_con))
+  expect_equal(get_fitted(mod_B), get_fitted(mod_B_con), tolerance = tol)
+  expect_equal(get_res(mod_B), get_res(mod_B_con), tolerance = tol)
+
+  mod_Z_con <- estimate_null(mod_Y, Cmat_F)
+  expect_equal(get_fitted(mod_Z), get_fitted(mod_Z_con), tolerance = tol)
+  expect_equal(get_res(mod_Z), get_res(mod_Z_con), tolerance = tol)
 
   oswald2013_mod <- oswald2013
   oswald2013_mod$X_null <- 2
 
   mod_G <- update(mod_C2, data = oswald2013_mod)
   mod_F_Xnull <- estimate_null(mod_G, Cmat_F)
-  expect_equal(get_fitted(mod_F), get_fitted(mod_F_Xnull))
-  expect_equal(get_res(mod_F), get_res(mod_F_Xnull))
+  expect_equal(get_fitted(mod_F), get_fitted(mod_F_Xnull), tolerance = tol)
+  expect_equal(get_res(mod_F), get_res(mod_F_Xnull), tolerance = tol)
 
 })
 
 test_that("get_cluster() works for rma.mv objects.", {
   study_fac <- as.factor(oswald2013$Study)
-  expect_equal(get_cluster(mod_A), study_fac)
-  expect_equal(get_cluster(mod_B), study_fac)
-  expect_equal(get_cluster(mod_C1), study_fac)
-  expect_equal(get_cluster(mod_C2), study_fac)
-  expect_equal(get_cluster(mod_D), study_fac)
-  expect_equal(get_cluster(mod_E), study_fac)
-  expect_equal(get_cluster(mod_F), study_fac)
+  expect_equal(get_cluster(mod_A), study_fac, tolerance = tol)
+  expect_equal(get_cluster(mod_B), study_fac, tolerance = tol)
+  expect_equal(get_cluster(mod_C1), study_fac, tolerance = tol)
+  expect_equal(get_cluster(mod_C2), study_fac, tolerance = tol)
+  expect_equal(get_cluster(mod_D), study_fac, tolerance = tol)
+  expect_equal(get_cluster(mod_E), study_fac, tolerance = tol)
+  expect_equal(get_cluster(mod_F), study_fac, tolerance = tol)
 })
 
 test_that("get_boot_F() works for rma.mv objects.", {
@@ -113,79 +127,86 @@ test_that("get_boot_F() works for rma.mv objects.", {
 
   expect_equal(
     get_boot_F(mod_A, y_boot = y_boot, C_mat = Cmat_int, cluster = get_cluster(mod_A)),
-    Wald_test(mod_A, constraints = Cmat_int, vcov = "CR0", test = "Naive-F")$Fstat
+    Wald_test(mod_A, constraints = Cmat_int, vcov = "CR0", test = "Naive-F")$Fstat, tolerance = tol
   )
 
   expect_equal(
     get_boot_F(mod_B, y_boot = y_boot, C_mat = Cmat_A, cluster = get_cluster(mod_B),
                type = "CR1", test = "HTZ"),
-    Wald_test(mod_B, constraints = Cmat_A, vcov = "CR1", test = "HTZ")$Fstat
+    Wald_test(mod_B, constraints = Cmat_A, vcov = "CR1", test = "HTZ")$Fstat, tolerance = tol
   )
 
   expect_equal(
     get_boot_F(mod_C1, y_boot = y_boot, C_mat = Cmat_B, cluster = get_cluster(mod_C1),
                type = "CR2", test = "EDT"),
-    Wald_test(mod_C1, constraints = Cmat_B, vcov = "CR2", test = "EDT")$Fstat
+    Wald_test(mod_C1, constraints = Cmat_B, vcov = "CR2", test = "EDT")$Fstat, tolerance = tol
   )
 
   expect_equal(
     get_boot_F(mod_C1, y_boot = y_boot, C_mat = Cmat_D, cluster = get_cluster(mod_C1),
                type = "CR3", test = "chi-sq"),
-    Wald_test(mod_C1, constraints = Cmat_D, vcov = "CR3", test = "chi-sq")$Fstat
+    Wald_test(mod_C1, constraints = Cmat_D, vcov = "CR3", test = "chi-sq")$Fstat, tolerance = tol
   )
 
   expect_equal(
     get_boot_F(mod_C2, y_boot = y_boot, C_mat = Cmat_E, cluster = get_cluster(mod_C2),
                type = "CR2", test = "Naive-Fp"),
-    Wald_test(mod_C2, constraints = Cmat_E, vcov = "CR2", test = "Naive-Fp")$Fstat
+    Wald_test(mod_C2, constraints = Cmat_E, vcov = "CR2", test = "Naive-Fp")$Fstat, tolerance = tol
   )
 
   expect_equal(
     get_boot_F(mod_C2, y_boot = y_boot, C_mat = Cmat_F, cluster = get_cluster(mod_C2),
                type = "CR0", test = "HTA"),
-    Wald_test(mod_C2, constraints = Cmat_F, vcov = "CR0", test = "HTA")$Fstat
+    Wald_test(mod_C2, constraints = Cmat_F, vcov = "CR0", test = "HTA")$Fstat, tolerance = tol
+  )
+
+  expect_equal(
+    get_boot_F(mod_Y, y_boot = y_boot, C_mat = Cmat_F, cluster = oswald2013$Study,
+               type = "CR0", test = "HTA"),
+    Wald_test(mod_Y, constraints = Cmat_F, cluster = oswald2013$Study,
+              vcov = "CR0", test = "HTA")$Fstat, tolerance = tol
   )
 
   # get_boot_F_f
 
   expect_equal(
     get_boot_F(mod_A, y_boot = y_boot, C_mat = Cmat_int, cluster = get_cluster(mod_A)),
-    get_boot_F_f(mod_A, C_mat = Cmat_int, cluster = get_cluster(mod_A))(y_boot)
+    get_boot_F_f(mod_A, C_mat = Cmat_int, cluster = get_cluster(mod_A))(y_boot), tolerance = tol
   )
 
   expect_equal(
     get_boot_F(mod_B, y_boot = y_boot, C_mat = Cmat_A, cluster = get_cluster(mod_B),
                type = "CR1", test = "HTZ"),
     get_boot_F_f(mod_B, C_mat = Cmat_A, cluster = get_cluster(mod_B),
-               type = "CR1", test = "HTZ")(y_boot)
+               type = "CR1", test = "HTZ")(y_boot), tolerance = tol
   )
 
   expect_equal(
     get_boot_F(mod_C1, y_boot = y_boot, C_mat = Cmat_B, cluster = get_cluster(mod_C1),
                type = "CR2", test = "EDT"),
     get_boot_F_f(mod_C1, C_mat = Cmat_B, cluster = get_cluster(mod_C1),
-               type = "CR2", test = "EDT")(y_boot)
+               type = "CR2", test = "EDT")(y_boot), tolerance = tol
   )
 
   expect_equal(
     get_boot_F(mod_C1, y_boot = y_boot, C_mat = Cmat_D, cluster = get_cluster(mod_C1),
                type = "CR3", test = "chi-sq"),
     get_boot_F_f(mod_C1, C_mat = Cmat_D, cluster = get_cluster(mod_C1),
-               type = "CR3", test = "chi-sq")(y_boot)
+               type = "CR3", test = "chi-sq")(y_boot), tolerance = tol
   )
 
   expect_equal(
     get_boot_F(mod_C2, y_boot = y_boot, C_mat = Cmat_E, cluster = get_cluster(mod_C2),
                type = "CR2", test = "Naive-Fp"),
     get_boot_F_f(mod_C2, C_mat = Cmat_E, cluster = get_cluster(mod_C2),
-               type = "CR2", test = "Naive-Fp")(y_boot)
+               type = "CR2", test = "Naive-Fp")(y_boot), tolerance = tol
   )
 
   expect_equal(
     get_boot_F(mod_C2, y_boot = y_boot, C_mat = Cmat_F, cluster = get_cluster(mod_C2),
                type = "CR0", test = "HTA"),
     get_boot_F_f(mod_C2, C_mat = Cmat_F, cluster = get_cluster(mod_C2),
-               type = "CR0", test = "HTA")(y_boot)
+               type = "CR0", test = "HTA")(y_boot), tolerance = tol
   )
 
 
@@ -200,7 +221,7 @@ test_that("get_boot_F() works for rma.mv objects.", {
                   data = oswald2013,
                   sparse = TRUE)
   expect_false(all(coef(mod_F) == coef(mod_G)))
-  expect_equal(coef(mod_G), coef(mod_H))
+  expect_equal(coef(mod_G), coef(mod_H), tolerance = tol)
   expect_false(all(y_boot == oswald2013$yi_boot))
 
   Cmat_G <- constrain_equal("Crit.Cat", reg_ex = TRUE, coefs = coef(mod_G))
@@ -210,9 +231,9 @@ test_that("get_boot_F() works for rma.mv objects.", {
   H_stat <- get_boot_F(mod_H, y_boot = y_boot, C_mat = Cmat_G, cluster = get_cluster(mod_G),
                        type = "CR0", test = "HTZ")
 
-  expect_equal(G_stat, F_stat)
+  expect_equal(G_stat, F_stat, tolerance = tol)
 
-  expect_equal(H_stat, F_stat)
+  expect_equal(H_stat, F_stat, tolerance = tol)
 
   expect_false(G_stat == Wald_test(mod_G, constraints = Cmat_G, vcov = "CR0", test = "HTZ")$Fstat)
 
@@ -248,7 +269,7 @@ compare_mod_results <- function(mod, scram, ord = 1:length(get_res(mod)), tol =
   expect_equal(coef(mod), coef(scram), tolerance = tol)
   expect_equal(as.numeric(get_res(mod)[ord]), as.numeric(get_res(scram)), tolerance = tol)
   expect_equal(as.numeric(get_fitted(mod)[ord]), as.numeric(get_fitted(scram)), tolerance = tol)
-  expect_equal(get_cluster(mod)[ord], get_cluster(scram))
+  expect_equal(get_cluster(mod)[ord], get_cluster(scram), tolerance = tol)
 }
 
 test_that("Wald_test_cwb() results do not depend on sort order.", {
@@ -407,8 +428,8 @@ test_that("Wald_test_cwb() works when rma.mv uses subset.", {
                             test = "Naive-F",
                             seed = 19)
 
-  expect_equal(attr(test_full, "original"), attr(test_sub, "original"))
-  expect_equal(attr(test_full, "bootstraps"), attr(test_sub, "bootstraps"))
+  expect_equal(attr(test_full, "original"), attr(test_sub, "original"), tolerance = tol)
+  expect_equal(attr(test_full, "bootstraps"), attr(test_sub, "bootstraps"), tolerance = tol)
 
 })
 
@@ -448,7 +469,7 @@ test_that("Wald_test_cwb() works with user-weighted rma.mv models.", {
   expect_s3_class(test_wt2, "Wald_test_wildmeta")
   expect_true(!is.na(test_wt2$p_val))
 
-  expect_equal(test_wt1, test_wt2)
+  expect_equal(test_wt1, test_wt2, tolerance = tol)
 
 })
 
